import java.awt.*;
import java.awt.event.*;
import javax.microedition.midlet.*;

public class PdaLander extends MIDlet {

    Frame frame = new Frame ("PDAlander");
    Simulation simulation;
    ScreenManager screenManager = new ScreenManager ();

    Label velocityDisplay = new Label ();
    Label heightDisplay = new Label ();
    Label fuelDisplay = new Label ();
    Scrollbar thrustSlider = 
        new Scrollbar (Scrollbar.HORIZONTAL, 0, 0, 0, 100);

    static final long GRAVITY = 1620; // mm/s2
    static final long ACCELLERATION = 2*GRAVITY;
    static final long MAX_VELOCITY = 1099;
    
    long velocity = 0; // mm/s 
    long height = 1000000; // mm    
    long fuel = 100000;     // ms
    long thrust = 0;


    class ScreenManager implements Runnable {
        
        public String milliToStr (long milli) {
            return (milli / 1000) + "." + Math.abs ((milli % 1000) / 100);
        } 

        public void run () {

            if (height <= 0) 
                heightDisplay.setText 
                    (velocity <= MAX_VELOCITY
                     ? "Landed"
                     : "Crashed!");
            else 
                heightDisplay.setText (milliToStr (height));

            velocityDisplay.setText (milliToStr (velocity));
            fuelDisplay.setText (milliToStr (fuel < 0 ? 0 : fuel));
                
            thrust = fuel <= 0 ? 0 : thrustSlider.getValue ();
        }
    }
    

    class Simulation extends Thread {
        
        long time = System.currentTimeMillis ();
        
        public void run () {  // 0..100
            
            do {
                long dt = System.currentTimeMillis () - time;
                
                velocity += ((GRAVITY - (ACCELLERATION * thrust) / 100) * dt) / 1000; 
                height -= (velocity * dt) / 1000;
                time  += dt;
                fuel -= (thrust * dt) / 100;
                
                try {
                    EventQueue.invokeAndWait (screenManager);
                }
                catch (Exception e) {
                    throw new RuntimeException (e.toString ());
                } 
            }
            while (height > 0);
        }
    }
    

    public PdaLander () {
        
        Panel intermediate = new Panel (new BorderLayout ());
        Panel controlPanel = new Panel (new GridLayout (0, 2));
        
        controlPanel.add (new Label ("Height:", Label.RIGHT));
        controlPanel.add (heightDisplay);
        controlPanel.add (new Label ("Velocity:", Label.RIGHT));
        controlPanel.add (velocityDisplay);
        controlPanel.add (new Label ("Fuel:", Label.RIGHT));
        controlPanel.add (fuelDisplay);
        controlPanel.add (new Label ("Thrust:", Label.RIGHT));
        controlPanel.add (thrustSlider);

        intermediate.add (controlPanel, BorderLayout.NORTH);
        frame.add (intermediate, BorderLayout.CENTER);
        frame.addWindowListener (new WindowAdapter () {
                public void windowClosing (WindowEvent ev) {
                    notifyDestroyed ();
                }
            });
        
        frame.pack ();
    }


    public void startApp () {
        frame.show ();
        if (simulation == null) {
            simulation = new Simulation ();
            simulation.start ();
        }
    }

    
    public void pauseApp() {
    }


    public void destroyApp (boolean unconditional) {
        height = -100000000;
        frame.dispose ();
    }   
}
