import java.awt.*;
import java.awt.event.*;
import javax.microedition.midlet.*;

public class PdaLander2 extends MIDlet {

    Frame frame = new Frame ("PDAlander");
    Simulation simulation;
    ScreenManager screenManager = new ScreenManager ();

    Label velocityDisplay = new Label ();
    Label heightDisplay = new Label ();
    Label fuelDisplay = new Label ();
    Scrollbar thrustSlider = 
        new Scrollbar (Scrollbar.HORIZONTAL, 0, 0, 0, 100);

    ExternalView externalView = new ExternalView ();

    static final long START_HEIGHT = 1000000;
    static final long GRAVITY = 1620; // mm/s2
    static final long ACCELLERATION = 2*GRAVITY;
    static final long MAX_VELOCITY = 1000;
    
    long velocity = 0; // mm/s 
    long height = START_HEIGHT; // mm    
    long fuel = 100000;     // ms
    long thrust = 0;


    class ExternalView extends Canvas {

        int oldY;
        long oldThrust;

        public void paint (Graphics g) {
            
            //    g.drawRect (0, 0, d.width, d.height);
            
            int x = getSize ().width / 2; 
            int y = getScrY ();
            
            g.drawLine (x-5, y-1, x+5, y-1);
            g.drawLine (x-5, y-1, x, y - 10);
            g.drawLine (x+5, y-1, x, y - 10);

            if (thrust > 10) {
                g.drawLine (x-3, y+1, x-3, (int) (y + 1 + thrust / 20));
                g.drawLine (x+3, y+1, x+3, (int) (y + 1 + thrust / 20));
                g.drawLine (x, y+1, x,  (int) (y + 1 + thrust / 15));
            }

            oldY = y;
            oldThrust = thrust;
        }

        public Dimension getPreferredSize () {
            return new Dimension (50, 100);
        }

        public Dimension getMinimumSize () {
            return new Dimension (20, 50);
        }


        int getScrY () {
            int scrH = getSize ().height;
            return scrH - (int) (height * scrH / START_HEIGHT);
        }
        
        public void check () {
            if (thrust != oldThrust || oldY != getScrY ())
                repaint ();
        }

    }


    class ScreenManager implements Runnable {
        
        public String milliToStr (long milli) {
            return (milli / 1000) + "." + Math.abs ((milli % 1000) / 100);
        } 

        public void run () {

            if (height <= 0) 
                heightDisplay.setText 
                    (velocity <= MAX_VELOCITY
                     ? "Landed"
                     : "Crashed!");
            else 
                heightDisplay.setText (milliToStr (height));
            
            velocityDisplay.setText (milliToStr (velocity));
            fuelDisplay.setText (milliToStr (fuel < 0 ? 0 : fuel));
            
            thrust = fuel <= 0 ? 0 : thrustSlider.getValue ();

            externalView.check ();
        }
    }
    
    
    class Simulation extends Thread {
        
        long time = System.currentTimeMillis ();
        
        public void run () {  // 0..100
            
            do {
                long dt = System.currentTimeMillis () - time;
                
                velocity += ((GRAVITY - (ACCELLERATION * thrust) / 100) * dt) / 1000; 
                height -= (velocity * dt) / 1000;
                time  += dt;
                fuel -= (thrust * dt) / 100;
                
                try {
                    EventQueue.invokeAndWait (screenManager);
                }
                catch (Exception e) {
                    throw new RuntimeException (e.toString ());
                } 
            }
            while (height > 0);
        }
    }
    
    
    public PdaLander2 () {
        
        Dimension d = Toolkit.getDefaultToolkit ().getScreenSize ();
        boolean vertical = d.height > d.width;

        Panel intermediate = new Panel (new BorderLayout ());
        Panel controlPanel = new Panel (new GridLayout (0, vertical ? 1 : 2));

        int align = vertical ? Label.LEFT : Label.RIGHT;
        
        controlPanel.add (new Label ("Height:", align));
        controlPanel.add (heightDisplay);
        controlPanel.add (new Label ("Velocity:", align));
        controlPanel.add (velocityDisplay);
        controlPanel.add (new Label ("Fuel:", align));
        controlPanel.add (fuelDisplay);
        controlPanel.add (new Label ("Thrust:", align));
        controlPanel.add (thrustSlider);

        intermediate.add (controlPanel, BorderLayout.NORTH);
        frame.add (intermediate, BorderLayout.WEST);
        frame.add (externalView, BorderLayout.CENTER);

        frame.addWindowListener (new WindowAdapter () {
                public void windowClosing (WindowEvent ev) {
                    notifyDestroyed ();
                }
            });
        
        frame.pack ();
    }


    public void startApp () {
        frame.show ();
        if (simulation == null) {
            simulation = new Simulation ();
            simulation.start ();
        }
    }

    public void pauseApp() {
    }

    public void destroyApp (boolean unconditional) {
        height = -100000000;
        frame.dispose ();
    }       
}
