import java.awt.*;
import java.awt.event.*;
import javax.microedition.pim.*;
import java.util.Hashtable;
import java.util.Vector;

public class ContactDialog
    extends Dialog
    implements ItemListener, ActionListener {

    Hashtable cards = new Hashtable();
    Contact contact;
    ContactList contactList;
    Panel cardPane = new Panel(new CardLayout());
    Choice typeChoice = new Choice();

    Button okButton = new Button("ok");
    Button cancelButton = new Button("cancel");

    boolean result = false;
    Vector fieldList = new Vector();

    class FieldInfo {
        int id;
        int type;
        TextField field;
    }

    class FieldPane extends Panel {
        Panel labels = new Panel(new GridLayout(0, 1));
        Panel fields = new Panel(new GridLayout(0, 1));
        int type;

        FieldPane(int type) {
            super(new BorderLayout());
            this.type = type;
            add(labels, BorderLayout.WEST);
            add(fields, BorderLayout.CENTER);
        }

        void addField(int id, int type) {

            FieldInfo info = new FieldInfo();
            info.id = id;
            info.field = new TextField(30);
            info.type = type;

            labels.add(new Label(contactList.getFieldLabel(id)));
            fields.add(info.field);
            fieldList.addElement(info);
        }

    }

    public void actionPerformed(ActionEvent ev) {

        result = ev.getSource() == okButton;
        hide();
    }

    void addSub(int id, String type, int typeId) {
        FieldPane fieldPane = (FieldPane) cards.get(type);
        if (fieldPane == null) {
            fieldPane = new FieldPane(typeId);
            typeChoice.add(type);
            cards.put(type, fieldPane);
            Panel compact = new Panel(new BorderLayout());
            compact.add(fieldPane, BorderLayout.NORTH);
            ScrollPane sp = new ScrollPane();
            sp.add(compact);
            cardPane.add(sp, type);
        }

        fieldPane.addField(id, typeId);
    }

    void addField(int id) {
        if (id == Contact.UID || id == Contact.REVISION)
            return;
        int dataType = contact.getDataType(id);
        if (dataType == PIMElement.STRING)
            addSub(id, "Main", -1);
        else if (dataType == PIMElement.STRING_TYPED) {
            int[] types = contactList.getSupportedTypes(id);

            for (int i = 0; i < types.length; i++) {
                switch (types[i]) {
                    case Contact.TYPE_HOME :
                        addSub(id, "Home", types[i]);
                        break;
                    case Contact.TYPE_HOME2 :
                        addSub(id, "Home 2", types[i]);
                        break;
                    case Contact.TYPE_WORK :
                        addSub(id, "Work", types[i]);
                        break;
                    case Contact.TYPE_WORK2 :
                        addSub(id, "Work 2", types[i]);
                        break;

					// add other types here
                }
            }
        }

        // other data types are ignored
    }

    public void itemStateChanged(ItemEvent e) {
        CardLayout cl = (CardLayout) (cardPane.getLayout());
        cl.show(cardPane, (String) e.getItem());
    }

    public ContactDialog(Frame frame, ContactList contactList) {

        super(frame, "Edit Contact", true);

        this.contactList = contactList;
        contact = contactList.createContact();

        add(typeChoice, BorderLayout.NORTH);
        add(cardPane, BorderLayout.CENTER);

        typeChoice.addItemListener(this);

        int[] ids = contactList.getSupportedFields();

        for (int i = 0; i < ids.length; i++)
            addField(ids[i]);

        Panel buttonPane = new Panel();

        buttonPane.add(okButton);
        buttonPane.add(cancelButton);

        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        add(buttonPane, BorderLayout.SOUTH);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                result = false;
                hide();
            }
        });

        pack();
    }

    public boolean edit(Contact contact, String title) {

        setTitle(title);

        this.contact = contact;
        result = false;

        for (int i = 0; i < fieldList.size(); i++) {
            FieldInfo info = (FieldInfo) fieldList.elementAt(i);
            String text =
                contact.getDataType(info.id) == PIMElement.STRING
                    ? contact.getString(info.id)
                    : contact.getTypedString(info.id, info.type);

            info.field.setText(text == null ? "" : text);
        }

        show();

        if (result) {
            for (int i = 0; i < fieldList.size(); i++) {
                FieldInfo info = (FieldInfo) fieldList.elementAt(i);
                String text = info.field.getText();
                if (text.equals(""))
                    text = null;
                if (contact.getDataType(info.id) == PIMElement.STRING)
                    contact.setString(info.id, text);
                else
                    contact.setTypedString(info.id, info.type, text);

                info.field.setText(text == null ? "" : text);
            }
            contact.commit();
        }

        return result;
    }
}
