import java.io.*;
import java.util.*;

public class DayLog {

    public static final int MIN_TIME = 6 * 60;
    public static final int MAX_TIME = 22 * 60;
    public static final int MIN_VALUE = 40;
    public static final int MAX_VALUE = 280;
    public static final int MIN_BORDER = 70;
    public static final int MAX_BORDER = 160;

    int date; // yymd per byte
    Vector entries = new Vector ();

    class Entry {
        int time; // minutes since midnight
        int value;
    }


    public static int parseTime (String time) {
        int cut = time.indexOf (':');
        if (cut==-1) 
            return Integer.parseInt (time) * 60;
        
        return Integer.parseInt (time.substring (0, cut)) * 60 
            + Integer.parseInt (time.substring (cut+1));
    }
    

    public DayLog (int date) {
        this.date = date;
     }


    public DayLog (byte [] data) throws IOException {
        
        DataInputStream dis = new DataInputStream 
            (new ByteArrayInputStream (data));

        date = dis.readInt ();
        int count = dis.readInt ();

        for (int i = 0; i < count; i++) {
            Entry entry = new Entry ();
            entry.time = dis.readInt ();
            entry.value = dis.readInt ();
            entries.addElement (entry);
        }

        dis.close ();
    }


    public int getDate () {
        return date;
    }


    public String getTitle () {
        return "" + (date >> 16) 
            + "-" + ((date >> 8) & 0x0ff) 
            + "-" + (date & 0x0ff);
    } 

    public int getCount () {
        return entries.size ();
    }


    public int getValue (int index) {
        return ((Entry) entries.elementAt (index)).value;
    }


    public int [] getXPoints (int size) {

        int [] x = new int [getCount ()];
        
        for (int i = 0; i < getCount (); i++) {
            x [i] = (getTime (i)-MIN_TIME) * size / (MAX_TIME-MIN_TIME);
        }

        return x;
    }


    public static int getY (int value, int size) {
        return - (value -MIN_VALUE) * size / (MAX_VALUE-MIN_VALUE);
    }

    public int [] getYPoints (int size) {

        int [] y = new int [getCount ()];
        
        for (int i = 0; i < getCount (); i++) {
            y [i] = getY (getValue (i), size);
        }

        return y;
    }


    public int getTime (int index) {
        return ((Entry) entries.elementAt (index)).time;
    }

    
    public void set (int minutes, int value) {
        
        Entry entry = new Entry ();
        entry.time = minutes;
        entry.value = value;

        for (int i = getCount ()-1; i >= 0; i--) {
            int minutesI = getTime (i);
            
            if (minutes <= minutesI) {
                if (minutes == minutesI) 
                    entries.setElementAt (entry, i);
                else 
                    entries.insertElementAt (entry, i+1);

                return;
            }
        }
                
        entries.insertElementAt (entry, 0);
    }


    public void remove (int time) {
        
        int bestDelta = 24;
        int bestIndex = -1;

        for (int i = 0; i < entries.size (); i++) {
            int delta = Math.abs (time - getTime (i));
            if (delta < bestDelta) {
                bestDelta = delta;
                bestIndex = i;
            }
        }
        
        if (bestIndex != -1) {
            entries.removeElementAt (bestIndex);
        }
    }

    
    public byte [] getByteArray () throws IOException {
        ByteArrayOutputStream bos = new ByteArrayOutputStream ();
        DataOutputStream dos = new DataOutputStream (bos);
        
        dos.writeInt (date);
        int size = entries.size ();
        dos.writeInt (size);
        for (int i = 0; i < size; i++) {
            dos.writeInt (getTime (i));
            dos.writeInt (getValue (i));
        }

        dos.close ();

        return bos.toByteArray ();
    }
}
