import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.rms.*;


import java.util.*;
import java.io.*;

public class BloodSugarMidp extends MIDlet implements CommandListener {

    LogStorage log;
    DayLog dayLog;
    Display display;

    Chart chart;
    ValueForm valueForm;
    DateForm dateForm;

    class Chart extends Canvas {
	
	public void paint (Graphics g) {
	    g.setColor (0x00FFFFFF);
	    g.fillRect (0, 0, getWidth(), getHeight());
	    g.setColor (0x00000000);	    

	    int charWidth = g.getFont ().charWidth ('0');
	    int xSpace = 3*charWidth;
	    int ySpace = g.getFont().getHeight ();
		  
	    int h = getHeight () - 2*ySpace;
	    int w = getWidth () - xSpace;
	    
	    String title = dayLog.getTitle ();
	    int tw = g.getFont().stringWidth (title);
	    g.drawString (title, getWidth () / 2 - tw / 2, 0, g.TOP|g.LEFT);
	    
	    g.drawString (""+DayLog.MAX_VALUE, 0, ySpace, g.TOP|g.LEFT);
	    g.drawString (""+DayLog.MIN_VALUE, charWidth, h, g.TOP|g.LEFT);
	    
	    g.drawRect (xSpace, ySpace, w-1, h);
	    
	    g.translate (xSpace, h);

	    
	    g.drawString (""+(DayLog.MIN_TIME/60), 0, ySpace, g.TOP|g.LEFT);
	    g.drawString (""+(DayLog.MAX_TIME+DayLog.MIN_TIME)/120, w/2-charWidth, ySpace, g.TOP|g.LEFT);
	    g.drawString (""+(DayLog.MAX_TIME/60), w-2*charWidth, ySpace, g.TOP|g.LEFT);
	    
	    int mnb = DayLog.getY (DayLog.MIN_BORDER, h);
	    int mxb = DayLog.getY (DayLog.MAX_BORDER, h);
	    
	    g.setColor (0x00FF0000);
	    g.drawLine (0, mnb, w + 2, mnb);
	    g.drawLine (0, mxb, w + 2, mxb);
	    g.setColor (0x000000FF);
	    

	    int[] x = dayLog.getXPoints (w);
	    int[] y = dayLog.getYPoints (h);
	    
	    int count = dayLog.getCount ();
	    
	    if (count == 1) {
		    g.drawArc (x[0] - 2, y[0] - 2, 4, 4, 0, 360);
	    } 
	    else {
		for (int i = 0; i < count - 1; i++) {
		    g.drawLine(x[i], y[i],
			       x[i+1], y[i+1]);
		    
		    g.drawArc (x[i] -2 , y[i] - 2, 4, 4, 0, 360);
		    g.drawArc (x[i+1] -2 , y[i+1] - 2, 4, 4, 0, 360);
		}
	    }
	}
    }

    class DateForm extends Form {
	
	DateField dateField = new DateField ("Log date:", DateField.DATE);
    
	public DateForm (CommandListener cmdListener) {
	    super ("Select");
	    append (dateField);
	    addCommand (okCommand);
	    setCommandListener (cmdListener);
	}
	
	public Date getDate () {
	    return dateField.getDate ();
	}
	
    }
    

    class ValueForm extends Form {
    
	TextField measuredMgDl = new TextField ("mg/dl", "", 3, TextField.NUMERIC);
	TextField measurementTime = new TextField ("Measured at:", "", 5, TextField.ANY);
    
   
	public ValueForm (CommandListener cmdListener) {
	    super ("Enter Value");
	    append (measuredMgDl);
	    append (measurementTime);
	    addCommand (okCommand);
	    setCommandListener (cmdListener);
	}


	public String getValue () {
	    return measuredMgDl.getString ();
	}
	
	
	public int getTime () {
	    return DayLog.parseTime (measurementTime.getString ());
	}
    }
    
    static final Command okCommand = new Command ("OK", Command.SCREEN, 1);
    static final Command exitCommand = new Command ("Exit", Command.SCREEN, 1);
    static final Command setCommand = new Command ("Set", Command.SCREEN, 2);
    static final Command removeCommand = new Command ("Remove", Command.SCREEN, 2);
    static final Command dateCommand = new Command ("Date...", Command.SCREEN, 2);
    
    public BloodSugarMidp () throws RecordStoreException, IOException {
	log = new LogStorage ();
	dayLog = log.getDayLog (LogStorage.dateToInt (new Date ()));

	valueForm = new ValueForm (this);
	dateForm = new DateForm (this);
	chart = new Chart ();
	
	chart.addCommand (exitCommand);
	chart.addCommand (setCommand);
	chart.addCommand (removeCommand);
	chart.addCommand (dateCommand);

	chart.setCommandListener (this);
    }


    public void startApp () {
	display = Display.getDisplay (this);
	display.setCurrent (chart);
    }


    public void pauseApp () {
    }

    
    public void destroyApp (boolean unconditional) {
    }


    public void commandAction (Command c, Displayable d) {
	
	if (c == exitCommand) {
	    notifyDestroyed();
	}
	else if (c == setCommand) {
	    display.setCurrent (valueForm);
	}
	
	else if (c == okCommand && d == valueForm) {
	    try {
		dayLog.set (valueForm.getTime (), 
			    Integer.parseInt (valueForm.getValue ()));
		log.storeDayLog (dayLog);
	    
		display.setCurrent (chart);
		chart.repaint ();
	    } 
	    catch (Exception e) {
		throw new RuntimeException ("Error writing Daylog...");
	    }
	}
	else if (c == okCommand && d == dateForm) {
	    
	    try {
		dayLog = log.getDayLog (LogStorage.dateToInt (dateForm.getDate ()));
		display.setCurrent (chart);		
	    }
	    catch (Exception e) {
		throw new RuntimeException ("Error writing Daylog...");
	    }
	}
	else if (c == removeCommand) {
	    try {
		dayLog.remove (valueForm.getTime ());
		log.storeDayLog (dayLog);
	    }
	    catch (Exception e) {
		throw new RuntimeException ("Error removing Daylog...");
	    }
	}
	else if (c == dateCommand) {
	    display.setCurrent (dateForm);
	}
    }
}









